@@ -14,6 +14,10 @@
 
 #include "tkInt.h"
 
+#ifdef _WIN32
+#include "tkWinInt.h"
+#endif
+
 /*
  * Forward declarations for functions defined later in this file:
  */
@@ -337,11 +341,14 @@ DoConfig(
     int nullValue;
 
     nullValue = 0;
-    if ((*value == 0) && (specPtr->specFlags & TK_CONFIG_NULL_OK)) {
+    if ((*value == 0) && (specPtr->specFlags & (TK_CONFIG_NULL_OK|32 /* TCL_NULL_OK */))) {
 	nullValue = 1;
     }
 
     do {
+	if (specPtr->offset < 0) {
+	    break;
+	}
 	ptr = widgRec + specPtr->offset;
 	switch (specPtr->type) {
 	case TK_CONFIG_BOOLEAN:
@@ -641,7 +648,7 @@ Tk_ConfigureInfo(
 		|| (specPtr->specFlags & hateFlags)) {
 	    continue;
 	}
-	if (specPtr->argvName == NULL) {
+	if ((specPtr->argvName == NULL) || (specPtr->offset < 0)) {
 	    continue;
 	}
 	list = FormatConfigInfo(interp, tkwin, specPtr, widgRec);
@@ -709,7 +716,7 @@ FormatConfigInfo(
     }
     result = Tcl_Merge(5, argv);
     if (freeProc != NULL) {
-	if ((freeProc == TCL_DYNAMIC) || (freeProc == (Tcl_FreeProc *) free)) {
+	if (freeProc == TCL_DYNAMIC) {
 	    ckfree((char *) argv[4]);
 	} else {
 	    freeProc((char *) argv[4]);
@@ -752,35 +759,39 @@ FormatConfigValue(
 				 * function to free the result, or NULL if
 				 * result is static. */
 {
-    const char *ptr, *result;
+    void *ptr;
+    const char *result;
 
     *freeProcPtr = NULL;
+    if (specPtr->offset < 0) {
+	return NULL;
+    }
     ptr = widgRec + specPtr->offset;
     result = "";
     switch (specPtr->type) {
     case TK_CONFIG_BOOLEAN:
-	if (*((int *) ptr) == 0) {
+	if (*((int *)ptr) == 0) {
 	    result = "0";
 	} else {
 	    result = "1";
 	}
 	break;
     case TK_CONFIG_INT:
-	sprintf(buffer, "%d", *((int *) ptr));
+	snprintf(buffer, 200, "%d", *((int *)ptr));
 	result = buffer;
 	break;
     case TK_CONFIG_DOUBLE:
-	Tcl_PrintDouble(interp, *((double *) ptr), buffer);
+	Tcl_PrintDouble(interp, *((double *)ptr), buffer);
 	result = buffer;
 	break;
     case TK_CONFIG_STRING:
-	result = (*(char **) ptr);
+	result = (*(char **)ptr);
 	if (result == NULL) {
 	    result = "";
 	}
 	break;
     case TK_CONFIG_UID: {
-	Tk_Uid uid = *((Tk_Uid *) ptr);
+	Tk_Uid uid = *((Tk_Uid *)ptr);
 
 	if (uid != NULL) {
 	    result = uid;
@@ -788,7 +799,7 @@ FormatConfigValue(
 	break;
     }
     case TK_CONFIG_COLOR: {
-	XColor *colorPtr = *((XColor **) ptr);
+	XColor *colorPtr = *((XColor **)ptr);
 
 	if (colorPtr != NULL) {
 	    result = Tk_NameOfColor(colorPtr);
@@ -796,7 +807,7 @@ FormatConfigValue(
 	break;
     }
     case TK_CONFIG_FONT: {
-	Tk_Font tkfont = *((Tk_Font *) ptr);
+	Tk_Font tkfont = *((Tk_Font *)ptr);
 
 	if (tkfont != NULL) {
 	    result = Tk_NameOfFont(tkfont);
@@ -804,7 +815,7 @@ FormatConfigValue(
 	break;
     }
     case TK_CONFIG_BITMAP: {
-	Pixmap pixmap = *((Pixmap *) ptr);
+	Pixmap pixmap = *((Pixmap *)ptr);
 
 	if (pixmap != None) {
 	    result = Tk_NameOfBitmap(Tk_Display(tkwin), pixmap);
@@ -812,7 +823,7 @@ FormatConfigValue(
 	break;
     }
     case TK_CONFIG_BORDER: {
-	Tk_3DBorder border = *((Tk_3DBorder *) ptr);
+	Tk_3DBorder border = *((Tk_3DBorder *)ptr);
 
 	if (border != NULL) {
 	    result = Tk_NameOf3DBorder(border);
@@ -820,11 +831,11 @@ FormatConfigValue(
 	break;
     }
     case TK_CONFIG_RELIEF:
-	result = Tk_NameOfRelief(*((int *) ptr));
+	result = Tk_NameOfRelief(*((int *)ptr));
 	break;
     case TK_CONFIG_CURSOR:
     case TK_CONFIG_ACTIVE_CURSOR: {
-	Tk_Cursor cursor = *((Tk_Cursor *) ptr);
+	Tk_Cursor cursor = *((Tk_Cursor *)ptr);
 
 	if (cursor != NULL) {
 	    result = Tk_NameOfCursor(Tk_Display(tkwin), cursor);
@@ -832,27 +843,27 @@ FormatConfigValue(
 	break;
     }
     case TK_CONFIG_JUSTIFY:
-	result = Tk_NameOfJustify(*((Tk_Justify *) ptr));
+	result = Tk_NameOfJustify(*((Tk_Justify *)ptr));
 	break;
     case TK_CONFIG_ANCHOR:
-	result = Tk_NameOfAnchor(*((Tk_Anchor *) ptr));
+	result = Tk_NameOfAnchor(*((Tk_Anchor *)ptr));
 	break;
     case TK_CONFIG_CAP_STYLE:
-	result = Tk_NameOfCapStyle(*((int *) ptr));
+	result = Tk_NameOfCapStyle(*((int *)ptr));
 	break;
     case TK_CONFIG_JOIN_STYLE:
-	result = Tk_NameOfJoinStyle(*((int *) ptr));
+	result = Tk_NameOfJoinStyle(*((int *)ptr));
 	break;
     case TK_CONFIG_PIXELS:
-	sprintf(buffer, "%d", *((int *) ptr));
+	snprintf(buffer, 200, "%d", *((int *)ptr));
 	result = buffer;
 	break;
     case TK_CONFIG_MM:
-	Tcl_PrintDouble(interp, *((double *) ptr), buffer);
+	Tcl_PrintDouble(interp, *((double *)ptr), buffer);
 	result = buffer;
 	break;
     case TK_CONFIG_WINDOW: {
-	tkwin = *((Tk_Window *) ptr);
+	tkwin = *((Tk_Window *)ptr);
 	if (tkwin != NULL) {
 	    result = Tk_PathName(tkwin);
 	}
@@ -928,7 +939,7 @@ Tk_ConfigureValue(
 	    &freeProc);
     Tcl_SetObjResult(interp, Tcl_NewStringObj(result, -1));
     if (freeProc != NULL) {
-	if ((freeProc == TCL_DYNAMIC) || (freeProc == (Tcl_FreeProc *) free)) {
+	if (freeProc == TCL_DYNAMIC) {
 	    ckfree((char *) result);
 	} else {
 	    freeProc((char *) result);
@@ -976,6 +987,9 @@ Tk_FreeOptions(
 	if ((specPtr->specFlags & needFlags) != needFlags) {
 	    continue;
 	}
+	if (specPtr->offset < 0) {
+	    continue;
+	}
 	ptr = widgRec + specPtr->offset;
 	switch (specPtr->type) {
 	case TK_CONFIG_STRING:
