@@ -15,6 +15,10 @@
 #include "tkText.h"
 #include "default.h"
 
+#ifdef _WIN32
+#include "tkWinInt.h"
+#endif
+
 /*
  * Index to use to select last character in line (very large integer):
  */
@@ -1128,7 +1132,7 @@ TkTextPrintIndex(
 	charIndex += numBytes;
     }
 
-    return sprintf(string, "%d.%d",
+    return snprintf(string, TK_POS_CHARS, "%d.%d",
 	    TkBTreeLinesTo(textPtr, indexPtr->linePtr) + 1, charIndex);
 }
 
@@ -2434,6 +2438,9 @@ StartEnd(
 		firstChar = 0;
 	    }
             if (offset == 0) {
+		if (indexPtr->byteIndex == 0) {
+		    goto done;
+		}
                 if (modifier == TKINDEX_DISPLAY) {
                     TkTextIndexBackChars(textPtr, indexPtr, 1, indexPtr,
                         COUNT_DISPLAY_INDICES);
@@ -2446,9 +2453,6 @@ StartEnd(
             }
             offset -= chSize;
 	    if (offset < 0) {
-		if (indexPtr->byteIndex == 0) {
-		    goto done;
-		}
 		segPtr = TkTextIndexToSeg(indexPtr, &offset);
 	    }
 	}
