@@ -9,8 +9,8 @@
 #include "tkInt.h"
 #include "ttkThemeInt.h"
 
-#define MAX(a,b) (a > b ? a : b)
-#define MIN(a,b) (a < b ? a : b)
+#define MAX(a,b) ((a) > (b) ? (a) : (b))
+#define MIN(a,b) ((a) < (b) ? (a) : (b))
 
 /*------------------------------------------------------------------------
  * +++ Ttk_Box and Ttk_Padding utilities:
@@ -31,6 +31,13 @@ Ttk_BoxContains(Ttk_Box box, int x, int y)
 	&& box.y <= y && y < box.y + box.height;
 }
 
+int
+TtkBoxEqual(Ttk_Box box1, Ttk_Box box2)
+{
+    return box1.x == box2.x && box1.y == box2.y
+	&& box1.width == box2.width && box1.height == box2.height;
+}
+
 Tcl_Obj *
 Ttk_NewBoxObj(Ttk_Box box)
 {
