@@ -55,7 +55,7 @@
 #define DEF_BUTTON_ANCHOR		"center"
 #define DEF_BUTTON_ACTIVE_BG_COLOR	ACTIVE_BG
 #define DEF_BUTTON_ACTIVE_BG_MONO	BLACK
-#define DEF_BUTTON_ACTIVE_FG_COLOR	WHITE
+#define DEF_BUTTON_ACTIVE_FG_COLOR	"systemPressedButtonTextColor"
 #define DEF_CHKRAD_ACTIVE_FG_COLOR	ACTIVE_FG
 #define DEF_BUTTON_ACTIVE_FG_MONO	WHITE
 #define DEF_BUTTON_BG_COLOR		NORMAL_BG
@@ -68,13 +68,14 @@
 #define DEF_BUTTON_DEFAULT		"disabled"
 #define DEF_BUTTON_DISABLED_FG_COLOR	DISABLED
 #define DEF_BUTTON_DISABLED_FG_MONO	""
-#define DEF_BUTTON_FG			NORMAL_FG
-#define DEF_CHKRAD_FG			DEF_BUTTON_FG
+#define DEF_BUTTON_FG			BLACK
+#define DEF_LABEL_FG			NORMAL_FG
+#define DEF_CHKRAD_FG			DEF_LABEL_FG
 #define DEF_BUTTON_FONT			"TkDefaultFont"
 #define DEF_BUTTON_HEIGHT		"0"
 #define DEF_BUTTON_HIGHLIGHT_BG_COLOR	DEF_BUTTON_BG_COLOR
 #define DEF_BUTTON_HIGHLIGHT_BG_MONO	DEF_BUTTON_BG_MONO
-#define DEF_BUTTON_HIGHLIGHT		"systemButtonFrame"
+#define DEF_BUTTON_HIGHLIGHT		NORMAL_FG
 #define DEF_LABEL_HIGHLIGHT_WIDTH	"0"
 //#if TK_MAC_BUTTON_USE_COMPATIBILITY_METRICS
 //#define DEF_BUTTON_HIGHLIGHT_WIDTH	"4"
@@ -124,7 +125,7 @@
 #define DEF_CANVAS_CURSOR		""
 #define DEF_CANVAS_HEIGHT		"7c"
 #define DEF_CANVAS_HIGHLIGHT_BG		NORMAL_BG
-#define DEF_CANVAS_HIGHLIGHT		BLACK
+#define DEF_CANVAS_HIGHLIGHT		NORMAL_FG
 #define DEF_CANVAS_HIGHLIGHT_WIDTH	"3"
 #define DEF_CANVAS_INSERT_BG		BLACK
 #define DEF_CANVAS_INSERT_BD_COLOR	"0"
@@ -174,7 +175,7 @@
 #define DEF_ENTRY_FONT			"TkTextFont"
 #define DEF_ENTRY_FG			NORMAL_FG
 #define DEF_ENTRY_HIGHLIGHT_BG		NORMAL_BG
-#define DEF_ENTRY_HIGHLIGHT		BLACK
+#define DEF_ENTRY_HIGHLIGHT		NORMAL_FG
 #define DEF_ENTRY_HIGHLIGHT_WIDTH	"3"
 #define DEF_ENTRY_INSERT_BG		NORMAL_FG
 #define DEF_ENTRY_INSERT_BD_COLOR	"0"
@@ -212,7 +213,7 @@
 #define DEF_FRAME_CURSOR		""
 #define DEF_FRAME_HEIGHT		"0"
 #define DEF_FRAME_HIGHLIGHT_BG		NORMAL_BG
-#define DEF_FRAME_HIGHLIGHT		BLACK
+#define DEF_FRAME_HIGHLIGHT		NORMAL_FG
 #define DEF_FRAME_HIGHLIGHT_WIDTH	"0"
 #define DEF_FRAME_PADX			"0"
 #define DEF_FRAME_PADY			"0"
@@ -248,7 +249,7 @@
 #define DEF_LISTBOX_FG			NORMAL_FG
 #define DEF_LISTBOX_HEIGHT		"10"
 #define DEF_LISTBOX_HIGHLIGHT_BG	NORMAL_BG
-#define DEF_LISTBOX_HIGHLIGHT		BLACK
+#define DEF_LISTBOX_HIGHLIGHT		NORMAL_FG
 #define DEF_LISTBOX_HIGHLIGHT_WIDTH	"0"
 #define DEF_LISTBOX_JUSTIFY		"left"
 #define DEF_LISTBOX_RELIEF		"solid"
@@ -343,7 +344,7 @@
 #define DEF_MENUBUTTON_HEIGHT		"0"
 #define DEF_MENUBUTTON_HIGHLIGHT_BG_COLOR DEF_MENUBUTTON_BG_COLOR
 #define DEF_MENUBUTTON_HIGHLIGHT_BG_MONO  DEF_MENUBUTTON_BG_MONO
-#define DEF_MENUBUTTON_HIGHLIGHT	BLACK
+#define DEF_MENUBUTTON_HIGHLIGHT	NORMAL_FG
 #define DEF_MENUBUTTON_HIGHLIGHT_WIDTH	"0"
 #define DEF_MENUBUTTON_IMAGE		NULL
 #define DEF_MENUBUTTON_INDICATOR	"1"
@@ -373,7 +374,7 @@
 #define DEF_MESSAGE_FG			NORMAL_FG
 #define DEF_MESSAGE_FONT		"TkDefaultFont"
 #define DEF_MESSAGE_HIGHLIGHT_BG	NORMAL_BG
-#define DEF_MESSAGE_HIGHLIGHT		BLACK
+#define DEF_MESSAGE_HIGHLIGHT		NORMAL_FG
 #define DEF_MESSAGE_HIGHLIGHT_WIDTH	"0"
 #define DEF_MESSAGE_JUSTIFY		"left"
 #define DEF_MESSAGE_PADX		"-1"
@@ -439,7 +440,7 @@
 #define DEF_SCALE_FROM			"0"
 #define DEF_SCALE_HIGHLIGHT_BG_COLOR	DEF_SCALE_BG_COLOR
 #define DEF_SCALE_HIGHLIGHT_BG_MONO	DEF_SCALE_BG_MONO
-#define DEF_SCALE_HIGHLIGHT		BLACK
+#define DEF_SCALE_HIGHLIGHT		NORMAL_FG
 #define DEF_SCALE_HIGHLIGHT_WIDTH	"0"
 #define DEF_SCALE_LABEL			""
 #define DEF_SCALE_LENGTH		"100"
@@ -474,7 +475,7 @@
 #define DEF_SCROLLBAR_CURSOR		""
 #define DEF_SCROLLBAR_EL_BORDER_WIDTH	"-1"
 #define DEF_SCROLLBAR_HIGHLIGHT_BG	NORMAL_BG
-#define DEF_SCROLLBAR_HIGHLIGHT		BLACK
+#define DEF_SCROLLBAR_HIGHLIGHT		NORMAL_FG
 #define DEF_SCROLLBAR_HIGHLIGHT_WIDTH	"0"
 #define DEF_SCROLLBAR_JUMP		"0"
 #define DEF_SCROLLBAR_ORIENT		"vertical"
@@ -501,7 +502,7 @@
 #define DEF_TEXT_FONT			"TkFixedFont"
 #define DEF_TEXT_HEIGHT			"24"
 #define DEF_TEXT_HIGHLIGHT_BG		NORMAL_BG
-#define DEF_TEXT_HIGHLIGHT		BLACK
+#define DEF_TEXT_HIGHLIGHT		NORMAL_FG
 #define DEF_TEXT_HIGHLIGHT_WIDTH	"3"
 #define DEF_TEXT_INSERT_BG		NORMAL_FG
 #define DEF_TEXT_INSERT_BD_COLOR	"0"
