@@ -4,10 +4,10 @@
  *	This file implements functions that decode & handle keyboard events on
  *	MacOS X.
  *
- * Copyright 2001-2009, Apple Inc.
- * Copyright (c) 2006-2009 Daniel A. Steffen <das@users.sourceforge.net>
- * Copyright (c) 2012 Adrian Robert.
- * Copyright 2015-2020 Marc Culler.
+ * Copyright © 2001-2009, Apple Inc.
+ * Copyright © 2006-2009 Daniel A. Steffen <das@users.sourceforge.net>
+ * Copyright © 2012 Adrian Robert.
+ * Copyright © 2015-2020 Marc Culler.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -47,7 +47,7 @@ static NSUInteger textInputModifiers;
 - (NSEvent *) tkProcessKeyEvent: (NSEvent *) theEvent
 {
 #ifdef TK_MAC_DEBUG_EVENTS
-    TKLog(@"-[%@(%p) %s] %@", [self class], self, _cmd, theEvent);
+    TKLog(@"-[%@(%p) %s] %@", [self class], self, sel_getName(_cmd), theEvent);
 #endif
     NSWindow *w = [theEvent window];
     TkWindow *winPtr = TkMacOSXGetTkWindow(w), *grabWinPtr, *focusWinPtr;
@@ -146,7 +146,7 @@ static NSUInteger textInputModifiers;
 
 #if defined(TK_MAC_DEBUG_EVENTS) || NS_KEYLOG == 1
 	TKLog(@"-[%@(%p) %s] repeat=%d mods=%x char=%x code=%lu c=%d type=%d",
-	      [self class], self, _cmd,
+	      [self class], self, sel_getName(_cmd),
 	      (type == NSKeyDown) && [theEvent isARepeat], modifiers, keychar,
 	      virt, w, type);
 #endif
@@ -400,6 +400,9 @@ static NSUInteger textInputModifiers;
 - (NSAttributedString *)attributedSubstringForProposedRange:(NSRange)theRange
       actualRange:(NSRangePointer)thePointer
 {
+    (void)theRange;
+    (void)thePointer;
+
     return nil;
 }
 
@@ -414,9 +417,13 @@ static NSUInteger textInputModifiers;
      replacementRange: (NSRange)repRange
 {
     TkWindow *winPtr = TkMacOSXGetTkWindow([self window]);
+    if (!winPtr) {
+	return;
+    }
     Tk_Window focusWin = (Tk_Window)winPtr->dispPtr->focusPtr;
     NSString *temp;
     NSString *str;
+    (void)selRange;
 
     str = ([aString isKindOfClass: [NSAttributedString class]]) ?
         [aString string] : aString;
@@ -493,6 +500,9 @@ static NSUInteger textInputModifiers;
 {
     NSRect rect;
     NSPoint pt;
+    (void)theRange;
+    (void)thePointer;
+
     pt.x = caret_x;
     pt.y = caret_y;
 
@@ -547,6 +557,8 @@ static NSUInteger textInputModifiers;
 
 - (NSUInteger)characterIndexForPoint: (NSPoint)thePoint
 {
+    (void)thePoint;
+
     if (NS_KEYLOG) {
 	TKLog(@"characterIndexForPoint request");
     }
@@ -556,6 +568,8 @@ static NSUInteger textInputModifiers;
 - (NSAttributedString *)attributedSubstringFromRange: (NSRange)theRange
 {
     static NSAttributedString *str = nil;
+    (void)theRange;
+
     if (str == nil) {
 	str = [NSAttributedString new];
     }
@@ -660,9 +674,12 @@ setXEventPoint(
 	    local.x -= contPtr->wmInfoPtr->xInParent;
 	    local.y -= contPtr->wmInfoPtr->yInParent;
 	} else {
-	    TkWindow *topPtr = TkMacOSXGetHostToplevel(winPtr)->winPtr;
-	    local.x -= (topPtr->wmInfoPtr->xInParent + contPtr->changes.x);
-	    local.y -= (topPtr->wmInfoPtr->yInParent + contPtr->changes.y);
+	    MacDrawable *topMacWin = TkMacOSXGetHostToplevel(winPtr);
+	    if (topMacWin) {
+		TkWindow *topPtr = topMacWin->winPtr;
+		local.x -= (topPtr->wmInfoPtr->xInParent + contPtr->changes.x);
+		local.y -= (topPtr->wmInfoPtr->yInParent + contPtr->changes.y);
+	    }
 	}
     } else if (winPtr->wmInfoPtr != NULL) {
 	local.x -= winPtr->wmInfoPtr->xInParent;
@@ -707,6 +724,10 @@ XGrabKeyboard(
 {
     keyboardGrabWinPtr = Tk_IdToWindow(display, grab_window);
     TkWindow *captureWinPtr = (TkWindow *) TkpGetCapture();
+    (void)owner_events;
+    (void)pointer_mode;
+    (void)keyboard_mode;
+    (void)time;
 
     if (keyboardGrabWinPtr && captureWinPtr) {
 	NSWindow *w = TkMacOSXGetNSWindowForDrawable(grab_window);
@@ -749,6 +770,9 @@ XUngrabKeyboard(
     Display* display,
     Time time)
 {
+    (void)display;
+    (void)time;
+
     if (modalSession) {
 	[NSApp endModalSession:modalSession];
 	modalSession = nil;
