@@ -97,7 +97,6 @@ FindSystemColor(
 {
     int l, u, r, i;
     int index;
-	int version = LOBYTE(LOWORD(GetVersion()));
 
     /*
      * Perform a binary search on the sorted array of colors.
@@ -120,15 +119,7 @@ FindSystemColor(
 	return 0;
     }
 
-    index = sysColors[i].index;
-    if (version < 4) {
-	if (index == COLOR_3DDKSHADOW) {
-	    index = COLOR_BTNSHADOW;
-	} else if (index == COLOR_3DLIGHT) {
-	    index = COLOR_BTNHIGHLIGHT;
-	}
-    }
-    *indexPtr = index;
+    *indexPtr = index = sysColors[i].index;
     colorPtr->pixel = GetSysColor(index);
 
     /*
@@ -182,7 +173,7 @@ TkpGetColor(
 	    && FindSystemColor(name+6, &color, &index))
 	    || TkParseColor(Tk_Display(tkwin), Tk_Colormap(tkwin), name,
 		    &color)) {
-	winColPtr = ckalloc(sizeof(WinColor));
+	winColPtr = (WinColor *)ckalloc(sizeof(WinColor));
 	winColPtr->info.color = color;
 	winColPtr->index = index;
 
@@ -220,7 +211,7 @@ TkpGetColorByValue(
     XColor *colorPtr)		/* Red, green, and blue fields indicate
 				 * desired color. */
 {
-    WinColor *tkColPtr = ckalloc(sizeof(WinColor));
+    WinColor *tkColPtr = (WinColor *)ckalloc(sizeof(WinColor));
 
     tkColPtr->info.color.red = colorPtr->red;
     tkColPtr->info.color.green = colorPtr->green;
@@ -283,7 +274,7 @@ int
 TkWinIndexOfColor(
     XColor *colorPtr)
 {
-    register WinColor *winColPtr = (WinColor *) colorPtr;
+    WinColor *winColPtr = (WinColor *) colorPtr;
     if (winColPtr->info.magic == COLOR_MAGIC) {
 	return winColPtr->index;
     }
@@ -309,7 +300,7 @@ TkWinIndexOfColor(
 
 int
 XAllocColor(
-    Display *display,
+    TCL_UNUSED(Display *),
     Colormap colormap,
     XColor *color)
 {
@@ -325,7 +316,8 @@ XAllocColor(
     if (GetDeviceCaps(dc, RASTERCAPS) & RC_PALETTE) {
 	unsigned long sizePalette = GetDeviceCaps(dc, SIZEPALETTE);
 	UINT newPixel, closePixel;
-	int new, refCount;
+	int isNew;
+	size_t refCount;
 	Tcl_HashEntry *entryPtr;
 	UINT index;
 
@@ -355,7 +347,7 @@ XAllocColor(
 		color->blue  = closeEntry.peBlue * 257;
 		entry = closeEntry;
 		if (index >= cmap->size) {
-		    OutputDebugStringA("XAllocColor: Colormap is bigger than we thought");
+		    OutputDebugStringW(L"XAllocColor: Colormap is bigger than we thought");
 		}
 	    } else {
 		cmap->size++;
@@ -366,11 +358,11 @@ XAllocColor(
 
 	color->pixel = PALETTERGB(entry.peRed, entry.peGreen, entry.peBlue);
 	entryPtr = Tcl_CreateHashEntry(&cmap->refCounts,
-		INT2PTR(color->pixel), &new);
-	if (new) {
+		INT2PTR(color->pixel), &isNew);
+	if (isNew) {
 	    refCount = 1;
 	} else {
-	    refCount = (PTR2INT(Tcl_GetHashValue(entryPtr))) + 1;
+	    refCount = (size_t)Tcl_GetHashValue(entryPtr) + 1;
 	}
 	Tcl_SetHashValue(entryPtr, INT2PTR(refCount));
     } else {
@@ -408,15 +400,16 @@ XAllocColor(
 
 int
 XFreeColors(
-    Display *display,
+    TCL_UNUSED(Display *),
     Colormap colormap,
     unsigned long *pixels,
     int npixels,
-    unsigned long planes)
+    TCL_UNUSED(unsigned long))
 {
     TkWinColormap *cmap = (TkWinColormap *) colormap;
     COLORREF cref;
-    UINT count, index, refCount;
+    UINT count, index;
+    size_t refCount;
     int i;
     PALETTEENTRY entry, *entries;
     Tcl_HashEntry *entryPtr;
@@ -436,14 +429,14 @@ XFreeColors(
 	    if (!entryPtr) {
 		Tcl_Panic("Tried to free a color that isn't allocated");
 	    }
-	    refCount = PTR2INT(Tcl_GetHashValue(entryPtr)) - 1;
+	    refCount = (size_t)Tcl_GetHashValue(entryPtr) - 1;
 	    if (refCount == 0) {
 		cref = pixels[i] & 0x00ffffff;
 		index = GetNearestPaletteIndex(cmap->palette, cref);
 		GetPaletteEntries(cmap->palette, index, 1, &entry);
 		if (cref == RGB(entry.peRed, entry.peGreen, entry.peBlue)) {
 		    count = cmap->size - index;
-		    entries = ckalloc(sizeof(PALETTEENTRY) * count);
+		    entries = (PALETTEENTRY *)ckalloc(sizeof(PALETTEENTRY) * count);
 		    GetPaletteEntries(cmap->palette, index+1, count, entries);
 		    SetPaletteEntries(cmap->palette, index, count, entries);
 		    ckfree(entries);
@@ -479,17 +472,17 @@ XFreeColors(
 
 Colormap
 XCreateColormap(
-    Display *display,
-    Window w,
-    Visual *visual,
-    int alloc)
+    TCL_UNUSED(Display *),
+    TCL_UNUSED(Window),
+    TCL_UNUSED(Visual *),
+    TCL_UNUSED(int))
 {
     char logPalBuf[sizeof(LOGPALETTE) + 256 * sizeof(PALETTEENTRY)];
     LOGPALETTE *logPalettePtr;
     PALETTEENTRY *entryPtr;
     TkWinColormap *cmap;
     Tcl_HashEntry *hashPtr;
-    int new;
+    int isNew;
     UINT i;
     HPALETTE sysPal;
 
@@ -503,7 +496,7 @@ XCreateColormap(
     logPalettePtr->palNumEntries = GetPaletteEntries(sysPal, 0, 256,
 	    logPalettePtr->palPalEntry);
 
-    cmap = ckalloc(sizeof(TkWinColormap));
+    cmap = (TkWinColormap *)ckalloc(sizeof(TkWinColormap));
     cmap->size = logPalettePtr->palNumEntries;
     cmap->stale = 0;
     cmap->palette = CreatePalette(logPalettePtr);
@@ -516,7 +509,7 @@ XCreateColormap(
     for (i = 0; i < logPalettePtr->palNumEntries; i++) {
 	entryPtr = logPalettePtr->palPalEntry + i;
 	hashPtr = Tcl_CreateHashEntry(&cmap->refCounts, INT2PTR(PALETTERGB(
-		entryPtr->peRed, entryPtr->peGreen, entryPtr->peBlue)), &new);
+		entryPtr->peRed, entryPtr->peGreen, entryPtr->peBlue)), &isNew);
 	Tcl_SetHashValue(hashPtr, INT2PTR(1));
     }
 
@@ -542,7 +535,7 @@ XCreateColormap(
 
 int
 XFreeColormap(
-    Display *display,
+    TCL_UNUSED(Display *),
     Colormap colormap)
 {
     TkWinColormap *cmap = (TkWinColormap *) colormap;
