@@ -22,7 +22,7 @@
  *	string representation of the option.
  *
  * Results:
- *	Returns a Tk_Uid that is the string identifier that identifies this
+ *	Returns a Tcl_Obj* with the string identifier that identifies this
  *	option. Returns NULL if there are no system defaults that match this
  *	pair.
  *
@@ -35,8 +35,8 @@
 Tcl_Obj *
 TkpGetSystemDefault(
     Tk_Window tkwin,		/* A window to use. */
-    CONST char *dbName,		/* The option database name. */
-    CONST char *className)	/* The name of the option class. */
+    const char *dbName,		/* The option database name. */
+    const char *className)	/* The name of the option class. */
 {
     Tcl_Obj *valueObjPtr;
     Tk_Uid classUid;
