@@ -369,7 +369,7 @@ XCopyPlane(
     HBRUSH bgBrush, fgBrush, oldBrush;
     TkpClipMask *clipPtr = (TkpClipMask*)gc->clip_mask;
 
-    display->request++;
+    LastKnownRequestProcessed(display)++;
 
     if (plane != 1) {
 	Tcl_Panic("Unexpected plane specified for XCopyPlane");
@@ -488,7 +488,7 @@ XCopyPlane(
  *
  * TkPutImage, XPutImage --
  *
- *	Copies a subimage from an in-memory image to a rectangle of of the
+ *	Copies a subimage from an in-memory image to a rectangle of the
  *	specified drawable.
  *
  * Results:
@@ -520,7 +520,7 @@ TkPutImage(
     HBITMAP bitmap;
     char *data;
 
-    display->request++;
+    LastKnownRequestProcessed(display)++;
 
     dc = TkWinGetDrawableDC(display, d, &state);
     SetROP2(dc, tkpWinRopModes[gc->function]);
@@ -1092,7 +1092,7 @@ XDrawArc(
     unsigned int width, unsigned int height,
     int start, int extent)
 {
-    display->request++;
+    LastKnownRequestProcessed(display)++;
 
     return DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, 0);
 }
@@ -1107,7 +1107,7 @@ XDrawArcs(
 {
     int ret = Success;
 
-    display->request++;
+    LastKnownRequestProcessed(display)++;
 
     while (narcs-- > 0) {
 	ret = DrawOrFillArc(display, d, gc, arcs[0].x, arcs[0].y,
@@ -1146,7 +1146,7 @@ XFillArc(
     unsigned int width, unsigned int height,
     int start, int extent)
 {
-    display->request++;
+    LastKnownRequestProcessed(display)++;
 
     return DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, 1);
 }
@@ -1161,7 +1161,7 @@ XFillArcs(
 {
     int ret = Success;
 
-    display->request++;
+    LastKnownRequestProcessed(display)++;
 
     while (narcs-- > 0) {
 	ret = DrawOrFillArc(display, d, gc, arcs[0].x, arcs[0].y,
